# coding: utf-8

# flake8: noqa

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from swagger_client.api.boards_api import BoardsApi
from swagger_client.api.datasets_api import DatasetsApi
from swagger_client.api.projects_api import ProjectsApi
from swagger_client.api.teams_api import TeamsApi
from swagger_client.api.users_api import UsersApi
# import ApiClient
from swagger_client.api_client import ApiClient
from swagger_client.configuration import Configuration
# import models into sdk package
from swagger_client.models.board import Board
from swagger_client.models.data_file import DataFile
from swagger_client.models.dataset import Dataset
from swagger_client.models.error_message import ErrorMessage
from swagger_client.models.patch_action import PatchAction
from swagger_client.models.project import Project
from swagger_client.models.project_sub import ProjectSub
from swagger_client.models.team import Team
from swagger_client.models.token import Token
from swagger_client.models.topic import Topic
from swagger_client.models.user import User



# coding: utf-8

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PatchAction(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'action': 'str',
        '_property': 'str',
        'value': 'object'
    }

    attribute_map = {
        'action': 'action',
        '_property': 'property',
        'value': 'value'
    }

    def __init__(self, action=None, _property=None, value=None):  # noqa: E501
        """PatchAction - a model defined in Swagger"""  # noqa: E501
        self._action = None
        self.__property = None
        self._value = None
        self.discriminator = None
        self.action = action
        self._property = _property
        if value is not None:
            self.value = value

    @property
    def action(self):
        """Gets the action of this PatchAction.  # noqa: E501

        Options: add/delete/update  # noqa: E501

        :return: The action of this PatchAction.  # noqa: E501
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this PatchAction.

        Options: add/delete/update  # noqa: E501

        :param action: The action of this PatchAction.  # noqa: E501
        :type: str
        """
        if action is None:
            raise ValueError("Invalid value for `action`, must not be `None`")  # noqa: E501

        self._action = action

    @property
    def _property(self):
        """Gets the _property of this PatchAction.  # noqa: E501


        :return: The _property of this PatchAction.  # noqa: E501
        :rtype: str
        """
        return self.__property

    @_property.setter
    def _property(self, _property):
        """Sets the _property of this PatchAction.


        :param _property: The _property of this PatchAction.  # noqa: E501
        :type: str
        """
        if _property is None:
            raise ValueError("Invalid value for `_property`, must not be `None`")  # noqa: E501

        self.__property = _property

    @property
    def value(self):
        """Gets the value of this PatchAction.  # noqa: E501


        :return: The value of this PatchAction.  # noqa: E501
        :rtype: object
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this PatchAction.


        :param value: The value of this PatchAction.  # noqa: E501
        :type: object
        """

        self._value = value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PatchAction, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PatchAction):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from swagger_client.api.boards_api import BoardsApi
from swagger_client.api.datasets_api import DatasetsApi
from swagger_client.api.projects_api import ProjectsApi
from swagger_client.api.teams_api import TeamsApi
from swagger_client.api.users_api import UsersApi



# coding: utf-8

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from swagger_client.api_client import ApiClient


class ProjectsApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def create_project(self, team_id, **kwargs):  # noqa: E501
        """Create Project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project(team_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param Project body: The **Project** object will be created.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_project_with_http_info(team_id, **kwargs)  # noqa: E501
        else:
            (data) = self.create_project_with_http_info(team_id, **kwargs)  # noqa: E501
            return data

    def create_project_with_http_info(self, team_id, **kwargs):  # noqa: E501
        """Create Project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_project_with_http_info(team_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param Project body: The **Project** object will be created.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['team_id', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'team_id' is set
        if ('team_id' not in params or
                params['team_id'] is None):
            raise ValueError("Missing the required parameter `team_id` when calling `create_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_id' in params:
            path_params['team_id'] = params['team_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['BearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_id}/projects', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_project(self, team_id, project_id, **kwargs):  # noqa: E501
        """Get Project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :return: Project
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_project_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_project_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
            return data

    def get_project_with_http_info(self, team_id, project_id, **kwargs):  # noqa: E501
        """Get Project  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project_with_http_info(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :return: Project
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['team_id', 'project_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_project" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'team_id' is set
        if ('team_id' not in params or
                params['team_id'] is None):
            raise ValueError("Missing the required parameter `team_id` when calling `get_project`")  # noqa: E501
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `get_project`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_id' in params:
            path_params['team_id'] = params['team_id']  # noqa: E501
        if 'project_id' in params:
            path_params['project_id'] = params['project_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['BearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_id}/projects/{project_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Project',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_project_subs(self, team_id, project_id, **kwargs):  # noqa: E501
        """Get Project Subs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project_subs(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :param bool current: Current login user
        :param bool expired: The results contains expired records or not
        :return: list[ProjectSub]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_project_subs_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_project_subs_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
            return data

    def get_project_subs_with_http_info(self, team_id, project_id, **kwargs):  # noqa: E501
        """Get Project Subs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_project_subs_with_http_info(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :param bool current: Current login user
        :param bool expired: The results contains expired records or not
        :return: list[ProjectSub]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['team_id', 'project_id', 'current', 'expired']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_project_subs" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'team_id' is set
        if ('team_id' not in params or
                params['team_id'] is None):
            raise ValueError("Missing the required parameter `team_id` when calling `get_project_subs`")  # noqa: E501
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `get_project_subs`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_id' in params:
            path_params['team_id'] = params['team_id']  # noqa: E501
        if 'project_id' in params:
            path_params['project_id'] = params['project_id']  # noqa: E501

        query_params = []
        if 'current' in params:
            query_params.append(('current', params['current']))  # noqa: E501
        if 'expired' in params:
            query_params.append(('expired', params['expired']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['BearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_id}/projects/{project_id}/subs', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ProjectSub]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_project_property(self, team_id, project_id, **kwargs):  # noqa: E501
        """Update Project property  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_project_property(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :param PatchAction body: PATCH Action
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_project_property_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.update_project_property_with_http_info(team_id, project_id, **kwargs)  # noqa: E501
            return data

    def update_project_property_with_http_info(self, team_id, project_id, **kwargs):  # noqa: E501
        """Update Project property  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_project_property_with_http_info(team_id, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str team_id: the **Team** identity (required)
        :param str project_id: the **Project** identity (required)
        :param PatchAction body: PATCH Action
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['team_id', 'project_id', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_project_property" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'team_id' is set
        if ('team_id' not in params or
                params['team_id'] is None):
            raise ValueError("Missing the required parameter `team_id` when calling `update_project_property`")  # noqa: E501
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `update_project_property`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'team_id' in params:
            path_params['team_id'] = params['team_id']  # noqa: E501
        if 'project_id' in params:
            path_params['project_id'] = params['project_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['BearerAuth']  # noqa: E501

        return self.api_client.call_api(
            '/teams/{team_id}/projects/{project_id}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



from logging import error
import requests
from requests import status_codes
import swagger_client as sw
import json
import pandas
import warnings
#warnings.filterwarnings('default')

SSL = True

class Project():
    def __init__(self, project, access_token, host=None, ssl=True):
        self.project = project.strip('/')
        s = self.project.split("/")
        if len(s) != 3:
            raise ValueError('invalid project')
        else:
            self.host = s[0]
            self.team_id = s[1]
            self.project_id = s[2]

        self.project = project
        self.access_token = access_token

        cfg = sw.Configuration()
        #cfg.api_key["token"] = access_token
        
        scheme = "https://" if ssl else "http://"
        cfg.host = scheme + (self.host if host is None else host) + "/api/v1"

        self.sw_client = sw.ApiClient(cfg, "Authorization", "Bearer "+access_token)

    def table(self, table_name):
        warnings.warn("Deprecated, use dataset instead.", DeprecationWarning)
        return Dataset(self.team_id, self.project_id, table_name, self.sw_client, self.access_token)

    def dataset(self, dataset_id):
        return Dataset(self.team_id, self.project_id, dataset_id, self.sw_client, self.access_token)
    
    def list_tables(self):
        warnings.warn("Deprecated, use list_datasets instead.", DeprecationWarning)
        return self.dataset(None).list()

    def list_datasets(self):
        return self.dataset.list()

class Dataset():
    def __init__(self, team_id, project_id, dataset_id, sw_client, token=None):
        self.team_id = team_id
        self.project_id = project_id
        self.dataset_id = dataset_id
        self.sw_client = sw_client
        self.api = sw.DatasetsApi(self.sw_client)
        self.token = token

    def list(self):
        return self.api.list_datasets(self.team_id, self.project_id)

    def get(self, get_schema=False):
        t = None
        try:
            t = self.api.get_dataset(self.team_id, self.project_id, self.dataset_id, schema=get_schema)
        except sw.rest.ApiException as e:
            if e.status != 404:
                _err_format(e)
        return t

    def exists(self):
        return self.get() is not None
        
    def create(self):
        try:
            self.api.update_dataset(self.team_id, self.project_id, self.dataset_id)
        except sw.rest.ApiException as e:
            _err_format(e)

    def delete(self):
        try:
            self.api.delete_dataset(self.team_id, self.project_id, self.dataset_id)
        except sw.rest.ApiException as e:
            _err_format(e)

    def update(self, prop, value):
        action = sw.PatchAction(action="UPDATE", _property=prop, value=value)
        try:
            self.api.update_dataset_property(self.team_id, self.project_id, self.dataset_id, body=action)
        except sw.rest.ApiException as e:
            _err_format(e)

    def update_schema(self, schema):
        self.update("schema", json.dumps(schema));
        
    def put(self, data, file_name=None, tag=None, content_type=None):
        if file_name is None or file_name.strip() == "":
            file_name = "0"
        if tag is None:
            tag = ""
        if content_type is None or content_type.strip() == "":
            content_type = ""
        try:
            self.api.put_dataset_data_file(self.team_id, self.project_id, self.dataset_id, file_name , x_di_tag=tag, content_type=content_type, body=data)
        except sw.rest.ApiException as e:
            _err_format(e)

    def put_csv(self, df, file_name=None, tag=None):
        body = df.to_csv()
        self.put(body, file_name, tag, "text/csv")
        
    def put_parquet(self, df, file_name=None, tag=None):
        body = df.to_parquet()
        self.put(body, file_name, tag, "application/parquet")

    def files(self):
        return self.api.list_dataset_data_files(self.team_id, self.project_id, self.dataset_id)

    def get_file_meta(self, data_file_name):
        return self.api.get_dataset_data_file_meta(self.team_id, self.project_id, self.dataset_id, data_file_name)

    def _get_file_url(self, file_name):
        url = '/'.join([self.sw_client.configuration.host, "teams", self.team_id, "projects", self.project_id, "datasets", self.dataset_id, "data", file_name])
        if self.token:
            url += "?token=" + self.token

        r = requests.get(url, allow_redirects=False)
        if r.status_code == 303:
            return r.headers.get('Location')
        else:
            raise BaseException("Get DataFile content failed: %s" % str(r.status_code))
    
    def _read_df(self, file_name, format):
        url = self._get_file_url(file_name)
        if url and format == "text/csv":
            return pandas.read_csv(url)
        elif format == "application/parquet":
            return pandas.read_parquet(url)
        else:
            raise BaseException("File format unsupported.")

    def read(self):
        dfs = []
        files = self.files()
        for f in files:
            df = self._read_df(f.name, f.content_type)
            dfs.append(df)
        return pandas.concat(dfs)
        
def Table(Dataset):
    def __init__(self, team_id, project_id, table_name, sw_client, token=None):
        Dataset.__init__(team_id, project_id, table_name, sw_client, token)
        warnings.warn("Deprecated, use Dataset instead.", DeprecationWarning)

def schema(df):
    columns = []
    for name in df.index.names:
        columns.append({
            "name": name,
            "data_type": str(df.index.dtype),
            "key": "index"
        })
    for index, value in df.dtypes.items():
        columns.append({
            "name": index,
            "data_type": str(value)
        })
    return {"columns": columns}

def _parse_identity(identity):
    s = identity.strip().split('/')
    if len(s) != 4:
        raise ValueError('invalid identity, it shold be like 42di.cn/your_id/your_project/your_table')
    project_id = '/'.join(s[:-1])
    dataset_id = s[-1:][0]
    return (project_id, dataset_id)

def put(identity, df, token, create=True, update_schema=False):
    project_id, dataset_id = _parse_identity(identity)
    project = Project(project_id, token, ssl=SSL)
    
    dataset = project.dataset(dataset_id)
    if not dataset.exists() and create:
        dataset.create()

    dataset.put_parquet(df)

    if update_schema:
        dataset.update_schema(schema(df))

def read(identity, token):
    project_id, dataset_id = _parse_identity(identity)
    project = Project(project_id, token, ssl=SSL)
    return project.dataset(dataset_id).read()

class DIException(Exception):
    def __init__(self, status, code, msg):
        self.status = status
        self.code = code
        self.msg = msg
        Exception.__init__(self, self.status, "HTTP Status: %s, Code: %s, Message: %s" % (self.status, self.code, self.msg))

def _err_format(e):
    err = {}
    try:
        err = json.loads(e.body)
    except json.decoder.JSONDecodeError as je:
        err = {"code": "JSONDecodeError", "message": je}
    
    raise DIException(e.status, err["code"], err["message"]) from None


# coding: utf-8

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProjectSub(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'team_id': 'str',
        'project_id': 'str',
        'subscriber': 'str',
        'expire_at': 'str',
        'created': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'team_id': 'team_id',
        'project_id': 'project_id',
        'subscriber': 'subscriber',
        'expire_at': 'expire_at',
        'created': 'created'
    }

    def __init__(self, id=None, team_id=None, project_id=None, subscriber=None, expire_at=None, created=None):  # noqa: E501
        """ProjectSub - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._team_id = None
        self._project_id = None
        self._subscriber = None
        self._expire_at = None
        self._created = None
        self.discriminator = None
        self.id = id
        self.team_id = team_id
        self.project_id = project_id
        self.subscriber = subscriber
        self.expire_at = expire_at
        self.created = created

    @property
    def id(self):
        """Gets the id of this ProjectSub.  # noqa: E501


        :return: The id of this ProjectSub.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ProjectSub.


        :param id: The id of this ProjectSub.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def team_id(self):
        """Gets the team_id of this ProjectSub.  # noqa: E501


        :return: The team_id of this ProjectSub.  # noqa: E501
        :rtype: str
        """
        return self._team_id

    @team_id.setter
    def team_id(self, team_id):
        """Sets the team_id of this ProjectSub.


        :param team_id: The team_id of this ProjectSub.  # noqa: E501
        :type: str
        """
        if team_id is None:
            raise ValueError("Invalid value for `team_id`, must not be `None`")  # noqa: E501

        self._team_id = team_id

    @property
    def project_id(self):
        """Gets the project_id of this ProjectSub.  # noqa: E501


        :return: The project_id of this ProjectSub.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this ProjectSub.


        :param project_id: The project_id of this ProjectSub.  # noqa: E501
        :type: str
        """
        if project_id is None:
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def subscriber(self):
        """Gets the subscriber of this ProjectSub.  # noqa: E501


        :return: The subscriber of this ProjectSub.  # noqa: E501
        :rtype: str
        """
        return self._subscriber

    @subscriber.setter
    def subscriber(self, subscriber):
        """Sets the subscriber of this ProjectSub.


        :param subscriber: The subscriber of this ProjectSub.  # noqa: E501
        :type: str
        """
        if subscriber is None:
            raise ValueError("Invalid value for `subscriber`, must not be `None`")  # noqa: E501

        self._subscriber = subscriber

    @property
    def expire_at(self):
        """Gets the expire_at of this ProjectSub.  # noqa: E501


        :return: The expire_at of this ProjectSub.  # noqa: E501
        :rtype: str
        """
        return self._expire_at

    @expire_at.setter
    def expire_at(self, expire_at):
        """Sets the expire_at of this ProjectSub.


        :param expire_at: The expire_at of this ProjectSub.  # noqa: E501
        :type: str
        """
        if expire_at is None:
            raise ValueError("Invalid value for `expire_at`, must not be `None`")  # noqa: E501

        self._expire_at = expire_at

    @property
    def created(self):
        """Gets the created of this ProjectSub.  # noqa: E501


        :return: The created of this ProjectSub.  # noqa: E501
        :rtype: datetime
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this ProjectSub.


        :param created: The created of this ProjectSub.  # noqa: E501
        :type: datetime
        """
        if created is None:
            raise ValueError("Invalid value for `created`, must not be `None`")  # noqa: E501

        self._created = created

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProjectSub, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectSub):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class DataFile(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'tag': 'str',
        'location': 'str',
        'content_type': 'str',
        'length': 'int',
        'creator': 'str',
        'created': 'datetime',
        'last_modified': 'datetime'
    }

    attribute_map = {
        'name': 'name',
        'tag': 'tag',
        'location': 'location',
        'content_type': 'content_type',
        'length': 'length',
        'creator': 'creator',
        'created': 'created',
        'last_modified': 'last_modified'
    }

    def __init__(self, name=None, tag=None, location=None, content_type=None, length=None, creator=None, created=None, last_modified=None):  # noqa: E501
        """DataFile - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._tag = None
        self._location = None
        self._content_type = None
        self._length = None
        self._creator = None
        self._created = None
        self._last_modified = None
        self.discriminator = None
        self.name = name
        if tag is not None:
            self.tag = tag
        if location is not None:
            self.location = location
        if content_type is not None:
            self.content_type = content_type
        if length is not None:
            self.length = length
        if creator is not None:
            self.creator = creator
        if created is not None:
            self.created = created
        if last_modified is not None:
            self.last_modified = last_modified

    @property
    def name(self):
        """Gets the name of this DataFile.  # noqa: E501


        :return: The name of this DataFile.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DataFile.


        :param name: The name of this DataFile.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def tag(self):
        """Gets the tag of this DataFile.  # noqa: E501


        :return: The tag of this DataFile.  # noqa: E501
        :rtype: str
        """
        return self._tag

    @tag.setter
    def tag(self, tag):
        """Sets the tag of this DataFile.


        :param tag: The tag of this DataFile.  # noqa: E501
        :type: str
        """

        self._tag = tag

    @property
    def location(self):
        """Gets the location of this DataFile.  # noqa: E501


        :return: The location of this DataFile.  # noqa: E501
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location):
        """Sets the location of this DataFile.


        :param location: The location of this DataFile.  # noqa: E501
        :type: str
        """

        self._location = location

    @property
    def content_type(self):
        """Gets the content_type of this DataFile.  # noqa: E501


        :return: The content_type of this DataFile.  # noqa: E501
        :rtype: str
        """
        return self._content_type

    @content_type.setter
    def content_type(self, content_type):
        """Sets the content_type of this DataFile.


        :param content_type: The content_type of this DataFile.  # noqa: E501
        :type: str
        """

        self._content_type = content_type

    @property
    def length(self):
        """Gets the length of this DataFile.  # noqa: E501


        :return: The length of this DataFile.  # noqa: E501
        :rtype: int
        """
        return self._length

    @length.setter
    def length(self, length):
        """Sets the length of this DataFile.


        :param length: The length of this DataFile.  # noqa: E501
        :type: int
        """

        self._length = length

    @property
    def creator(self):
        """Gets the creator of this DataFile.  # noqa: E501


        :return: The creator of this DataFile.  # noqa: E501
        :rtype: str
        """
        return self._creator

    @creator.setter
    def creator(self, creator):
        """Sets the creator of this DataFile.


        :param creator: The creator of this DataFile.  # noqa: E501
        :type: str
        """

        self._creator = creator

    @property
    def created(self):
        """Gets the created of this DataFile.  # noqa: E501


        :return: The created of this DataFile.  # noqa: E501
        :rtype: datetime
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this DataFile.


        :param created: The created of this DataFile.  # noqa: E501
        :type: datetime
        """

        self._created = created

    @property
    def last_modified(self):
        """Gets the last_modified of this DataFile.  # noqa: E501


        :return: The last_modified of this DataFile.  # noqa: E501
        :rtype: datetime
        """
        return self._last_modified

    @last_modified.setter
    def last_modified(self, last_modified):
        """Sets the last_modified of this DataFile.


        :param last_modified: The last_modified of this DataFile.  # noqa: E501
        :type: datetime
        """

        self._last_modified = last_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(DataFile, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DataFile):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    42di Python SDK

    No description provided
"""

from setuptools import setup, find_packages  # noqa: H301

NAME = "42di"
VERSION = "0.2"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil"]

setup(
    name=NAME,
    version=VERSION,
    description="42di API Reference",
    author_email="support@42di.com",
    url="https://42di.com",
    keywords=["42DI", "42di Python SDK"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description=" 42di.com "
)



# coding: utf-8

"""
    42di API Reference

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    Contact: support@42docs.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Dataset(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'team_id': 'str',
        'project_id': 'str',
        'name': 'str',
        'description': 'str',
        'schema': 'str',
        'schema_extra': 'str',
        'owners': 'list[str]',
        'creator': 'str',
        'created': 'datetime',
        'last_modified': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'team_id': 'team_id',
        'project_id': 'project_id',
        'name': 'name',
        'description': 'description',
        'schema': 'schema',
        'schema_extra': 'schema_extra',
        'owners': 'owners',
        'creator': 'creator',
        'created': 'created',
        'last_modified': 'last_modified'
    }

    def __init__(self, id=None, team_id=None, project_id=None, name=None, description=None, schema=None, schema_extra=None, owners=None, creator=None, created=None, last_modified=None):  # noqa: E501
        """Dataset - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._team_id = None
        self._project_id = None
        self._name = None
        self._description = None
        self._schema = None
        self._schema_extra = None
        self._owners = None
        self._creator = None
        self._created = None
        self._last_modified = None
        self.discriminator = None
        self.id = id
        if team_id is not None:
            self.team_id = team_id
        self.project_id = project_id
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if schema is not None:
            self.schema = schema
        if schema_extra is not None:
            self.schema_extra = schema_extra
        if owners is not None:
            self.owners = owners
        if creator is not None:
            self.creator = creator
        if created is not None:
            self.created = created
        if last_modified is not None:
            self.last_modified = last_modified

    @property
    def id(self):
        """Gets the id of this Dataset.  # noqa: E501


        :return: The id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Dataset.


        :param id: The id of this Dataset.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def team_id(self):
        """Gets the team_id of this Dataset.  # noqa: E501


        :return: The team_id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._team_id

    @team_id.setter
    def team_id(self, team_id):
        """Sets the team_id of this Dataset.


        :param team_id: The team_id of this Dataset.  # noqa: E501
        :type: str
        """

        self._team_id = team_id

    @property
    def project_id(self):
        """Gets the project_id of this Dataset.  # noqa: E501


        :return: The project_id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this Dataset.


        :param project_id: The project_id of this Dataset.  # noqa: E501
        :type: str
        """
        if project_id is None:
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def name(self):
        """Gets the name of this Dataset.  # noqa: E501


        :return: The name of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Dataset.


        :param name: The name of this Dataset.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this Dataset.  # noqa: E501


        :return: The description of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Dataset.


        :param description: The description of this Dataset.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def schema(self):
        """Gets the schema of this Dataset.  # noqa: E501


        :return: The schema of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._schema

    @schema.setter
    def schema(self, schema):
        """Sets the schema of this Dataset.


        :param schema: The schema of this Dataset.  # noqa: E501
        :type: str
        """

        self._schema = schema

    @property
    def schema_extra(self):
        """Gets the schema_extra of this Dataset.  # noqa: E501


        :return: The schema_extra of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._schema_extra

    @schema_extra.setter
    def schema_extra(self, schema_extra):
        """Sets the schema_extra of this Dataset.


        :param schema_extra: The schema_extra of this Dataset.  # noqa: E501
        :type: str
        """

        self._schema_extra = schema_extra

    @property
    def owners(self):
        """Gets the owners of this Dataset.  # noqa: E501


        :return: The owners of this Dataset.  # noqa: E501
        :rtype: list[str]
        """
        return self._owners

    @owners.setter
    def owners(self, owners):
        """Sets the owners of this Dataset.


        :param owners: The owners of this Dataset.  # noqa: E501
        :type: list[str]
        """

        self._owners = owners

    @property
    def creator(self):
        """Gets the creator of this Dataset.  # noqa: E501


        :return: The creator of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._creator

    @creator.setter
    def creator(self, creator):
        """Sets the creator of this Dataset.


        :param creator: The creator of this Dataset.  # noqa: E501
        :type: str
        """

        self._creator = creator

    @property
    def created(self):
        """Gets the created of this Dataset.  # noqa: E501


        :return: The created of this Dataset.  # noqa: E501
        :rtype: datetime
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this Dataset.


        :param created: The created of this Dataset.  # noqa: E501
        :type: datetime
        """

        self._created = created

    @property
    def last_modified(self):
        """Gets the last_modified of this Dataset.  # noqa: E501


        :return: The last_modified of this Dataset.  # noqa: E501
        :rtype: datetime
        """
        return self._last_modified

    @last_modified.setter
    def last_modified(self, last_modified):
        """Sets the last_modified of this Dataset.


        :param last_modified: The last_modified of this Dataset.  # noqa: E501
        :type: datetime
        """

        self._last_modified = last_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Dataset, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Dataset):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
